; +-----------------------+
; | PureHTML Highlighting |
; +-----------------------+

;-
;- Constants

; File IDs
#PHF_In  = 0
#PHF_Out = 1

; Library IDs
#PHL = 0



; Library file patterns
CompilerSelect (#PB_Compiler_OS)
  CompilerCase (#PB_OS_Windows)
    #PH_LibFile    = "SyntaxHilighting.dll"
    #PH_LibPattern = "DLL Files|*.dll|All Files (*.*)|*.*"
  CompilerCase (#PB_OS_MacOS)
    ;? #PH_LibFile = 
CompilerEndSelect




;-
;- Procedures


; Search for the syntax highlighter library
Procedure.i FindLibrary()
  Protected Result.i = #Null
  Protected PBHome.s = ""
  Protected File.s   = ""
  Protected i.i, n.i
  
  For i = 0 To CountProgramParameters() - 2
    If (LCase(ProgramParameter(i)) = "-lib")
      Result = OpenLibrary(#PHL, Trim(ProgramParameter(i+1), #DQUOTE$))
      If (Result)
        Break
      EndIf
    EndIf
  Next
  If (Not Result)
    Result = OpenLibrary(#PHL, #PH_LibFile)
  EndIf
  If (Result)
    Config\LibFile = GetCurrentDirectory() + #PH_LibFile
  Else
    PBHome = GetEnvironmentVariable("PUREBASIC_HOME")
    If (PBHome)
      File = PBHome + "SDK" + #PS$ + "Syntax Highlighting" + #PS$ + #PH_LibFile
      Result = OpenLibrary(#PHL, File)
    EndIf
    If (Result)
      Config\LibFile = File
    Else
      File = OpenFileRequester("Syntax Highlighting Library", GetCurrentDirectory(), #PH_LibPattern, 0)
      If (File)
        Result = OpenLibrary(#PHL, File)
      EndIf
    EndIf
  EndIf
  
  ProcedureReturn (Result)
EndProcedure





; Format an integer as a CSS color
Procedure.s CSSColor(Color.i)
  Color = RGB(Blue(Color), Green(Color), Red(Color))
  ProcedureReturn ("#" + RSet(Hex(Color), 6, "0"))
EndProcedure






; Encode HTML text safely
Procedure.s EncodeHTML(Input.s)
  
  ; Escape ampersands (must be done first!)
  Input = ReplaceString(Input, "&", "&amp;")
  
  ; Replace whitespace with non-breaking spaces
  Input = ReplaceString(Input, #TAB$, RSet("", Config\TabLength, " "))
  CompilerIf (Not #PH_CompactSpaces)
    Input = ReplaceString(Input, "  ", "&nbsp;&nbsp;")
    Input = ReplaceString(Input, "&nbsp; ", "&nbsp;&nbsp;")
  CompilerEndIf
  
  ; Escape HTML brackets
  Input = ReplaceString(Input, "<", "&lt;")
  Input = ReplaceString(Input, ">", "&gt;")
  
  ; Escape all newlines
  CompilerIf (#PH_CompactSpaces)
    Input = ReplaceString(Input, #CRLF$, #LF$)
    Input = ReplaceString(Input, #CR$,   #LF$)
  CompilerElse
    Input = ReplaceString(Input, #CRLF$, "<br/>")
    Input = ReplaceString(Input, #CR$, "<br/>")
    Input = ReplaceString(Input, #LF$, "<br/>")
    Input = ReplaceString(Input, "<br/>", "<br/>" + #LF$)
  CompilerEndIf
  
  ProcedureReturn (Input)
EndProcedure


; Check if "string" starts with "prefix" (case insensitive)
Procedure.i Starts(String.s, Prefix.s)
  If (LCase(Left(String, Len(Prefix))) = LCase(Prefix))
    ProcedureReturn (#True)
  Else
    ProcedureReturn (#False)
  EndIf
EndProcedure





; Callback used by the syntax highlighter library (writes out HTML data)
Procedure HTMLCallback(*Position, Length.i, Type.i)
  Static Raw.s, Breaks.i, Prefix.s
  Static PrevLine.i, PreWhite.s, PostWhite.s, IncludeNext.i, IncludeSuffix.i
  Static i.i, j.i, Link.s, LFIndex.i, HTML.s
  
  Raw = PeekS(*Position, Length)
  
  If (Config\HideIDE)
    If (Left(Raw, 16) = "; IDE Options = ")
      Config\Hidden = #True
    EndIf
  EndIf
  
  If (Length And (Not Config\Hidden))
    HTML = EncodeHTML(Raw)
    
    LFIndex = FindString(HTML, #LF$, 1)
    If (LFIndex)
      Raw = Mid(HTML, 1, LFIndex)
    Else
      Raw = HTML
    EndIf
    
    While (Raw)
    
      PreWhite = ""
      Repeat
        If (Left(Raw, 6) = "&nbsp;")
          Raw = Mid(Raw, 7)
          PreWhite + "&nbsp;"
        ElseIf (Left(Raw, 1) = " ")
          Raw = Mid(Raw, 2)
          PreWhite + " "
        Else
          Break
        EndIf
      ForEver
    
      PostWhite = ""
      Repeat
        If (Right(Raw, 6) = "&nbsp;")
          Raw = Left(Raw, Len(Raw) - 6)
          PostWhite + "&nbsp;"
        ElseIf (Right(Raw, 1) = " ")
          Raw = Left(Raw, Len(Raw) - 1)
          PostWhite + " "
        Else
          Break
        EndIf
      ForEver
      
      If (Config\LineCount = 0)
        PrevLine         = 0
        IncludeNext      = 0
        Config\LineCount = 1
      EndIf
      
      If (Config\IncludeLinks)
        If (Starts(Raw, "includefile") Or Starts(Raw, "xincludefile"))
          IncludeNext = 2
          IncludeSuffix = #True
        ElseIf (Starts(Raw, "includepath") Or Starts(Raw, "includebinary"))
          IncludeNext = 2
          IncludeSuffix = #False
        EndIf
      EndIf
      
      If (Config\LineNumbers)
        If (Config\LineCount <> PrevLine)
          Prefix = RSet(Str(Config\LineCount), Config\LineDigits + 1, " ") + " "
          CompilerIf (Not #PH_CompactSpaces)
            Prefix = ReplaceString(Prefix, " ", "&nbsp;")
          CompilerEndIf
          Prefix = "<span class='" + StyleRef(#PHS_LineNumber) + "'>" + Prefix + "</span>"
          WriteString(#PHF_Out, Prefix)
          PrevLine = Config\LineCount
        EndIf
        Breaks = CountString(Raw, #LF$)
        Config\LineCount + Breaks
      EndIf
      
      WriteString(#PHF_Out, PreWhite)
      If (Raw)
        WriteString(#PHF_Out, "<span class='" + StyleRef(Type) + "'>")
        If (IncludeNext = 1)
          i = FindString(Raw, #DQUOTE$, 1)
          If (i)
            j = FindString(Raw, #DQUOTE$, i + 1)
          EndIf
          If (Not (i And j))
            IncludeNext = 0
          EndIf
        EndIf
        If (IncludeNext = 1)
          WriteString(#PHF_Out, Mid(Raw, 1, i))
          Link = Mid(Raw, i+1, j - i - 1)
          WriteString(#PHF_Out, "<a href='" + EncodeHTML(Link))
          If (IncludeSuffix)
            WriteString(#PHF_Out, "." + GetExtensionPart(Config\OutFile))
          EndIf
          WriteString(#PHF_Out, "'>" + Link)
          WriteString(#PHF_Out, "</a>")
          WriteString(#PHF_Out, Mid(Raw, j))
          IncludeNext = 0
        Else
          If (IncludeNext = 2)
            IncludeNext = 1
          EndIf
          WriteString(#PHF_Out, Raw)
        EndIf
        WriteString(#PHF_Out, "</span>")
      EndIf
      WriteString(#PHF_Out, PostWhite)
      
      If (LFIndex)
        HTML = Mid(HTML, LFIndex + 1)
        LFIndex = FindString(HTML, #LF$, 1)
        If (LFIndex)
          Raw = Mid(HTML, 1, LFIndex)
        Else
          Raw = HTML
        EndIf
      Else
        Raw = ""
      EndIf
      
    Wend
  EndIf
EndProcedure


; Count the number of lines in a text buffer
Procedure CountLines(*Buffer)
  Protected *C.CHARACTER, CRs.i, LFs.i
  
  If (*Buffer)
    *C = *Buffer
    While (*C\c)
      If (*C\c = #LF)
        LFs + 1
      ElseIf (*C\c = #CR)
        CRs + 1
      EndIf
      *C + SizeOf(CHARACTER)
    Wend
    
    If (LFs > 0)
      Config\LineTotal = LFs
    Else
      Config\LineTotal = CRs
    EndIf
    If (Config\LineTotal)
      Config\LineDigits = Int(Log10(Config\LineTotal)) + 1
    Else
      Config\LineDigits = 1
    EndIf
    
  EndIf
EndProcedure



; Syntax-highlight a specific source file, save as the specified HTML file
Procedure.i HighlightFile(Input.s, Output.s)
  Protected Result.i = #False
  Protected *InBuffer, InLength.i
  Protected CSS.s, i.i, Temp.s
  
  If (Input And ReadFile(#PHF_In, Input))
    If (Output And CreateFile(#PHF_Out, Output))
      InLength = Lof(#PHF_In)
      If (InLength > 0)
        *InBuffer = AllocateMemory(InLength + 8)
        If (*InBuffer)
          If (ReadData(#PHF_In, *InBuffer, InLength))
            Config\LineCount = 0
            Config\Hidden = #False
            CountLines(*InBuffer)
            
            ; Generate body CSS style
            If (Config\LineNumbers)
              CSS = "  margin:0px;" + #LF$
            Else
              CSS = ""
            EndIf
            CSS + "  background-color:" + CSSColor(Config\BGColor) + ";" + #LF$
            CSS + "  font-family:" + Config\FontName + ",monospace;" + #LF$
            CSS + "  font-size:" + Str(Config\FontSize) + "pt;" + #LF$
            CompilerIf (#PH_CompactSpaces)
              CSS + "  white-space:pre;" + #LF$
            CompilerEndIf
            
            If (Config\FullPage)
            
              ; Generate HTML header
              WriteString(#PHF_Out, "<!DOCTYPE html PUBLIC " + #DQUOTE$ + "-//W3C//DTD XHTML 1.0 Transitional//EN" + #DQUOTE$ + " " + #LF$)
              WriteString(#PHF_Out, "  " + #DQUOTE$ + "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" + #DQUOTE$ + ">" + #LF$)
              WriteString(#PHF_Out, "<html>" + #LF$)
              WriteString(#PHF_Out, "<head>" + #LF$)
              ;WriteString(#PHF_Out, "<!ENTITY sp &nbsp;>" + #LF$)
              WriteString(#PHF_Out, "<title>" + EncodeHTML(GetFilePart(Input)) + "</title>" + #LF$ + #LF$)
              
              ; Generate a comment header
              WriteString(#PHF_Out, "<!-- Generated using " + #PH_Title + " " + VersionString(#PH_Version) + " -->" + #LF$)
              WriteString(#PHF_Out, "<!-- Original file: " + GetFilePart(Input) + " -->" + #LF$)
              WriteString(#PHF_Out, "<!-- Date: " + FormatDate("%mm/%dd/%yyyy, %hh:%ii", Date()) + " -->" + #LF$ + #LF$)
              
              ; Generate CSS style table
              WriteString(#PHF_Out, "<style type='text/css'>" + #LF$)
              WriteString(#PHF_Out, "body {" + #LF$ + CSS + "}" + #LF$)
              For i = 0 To #PHS_Count - 1
                WriteString(#PHF_Out, "span." + StyleRef(i) + " {" + #LF$)
                WriteString(#PHF_Out, "  color:" + CSSColor(Style(i)\Color) + ";" + #LF$)
                If (i = #PHS_LineNumber)
                  WriteString(#PHF_Out, "  background-color:" + CSSColor(Config\LNColor) + ";" + #LF$)
                EndIf
                If (Style(i)\Bold)
                  WriteString(#PHF_Out, "  font-weight:bold;" + #LF$)
                EndIf
                If (Style(i)\Italic)
                  WriteString(#PHF_Out, "  font-style:italic;" + #LF$)
                EndIf
                If (Style(i)\Underline)
                  Temp = "underline,"
                Else
                  Temp = ""
                EndIf
                If (Style(i)\Strike)
                  Temp + "line-through,"
                EndIf
                If (Temp)
                  WriteString(#PHF_Out, "  text-decoration:" + Trim(Temp, ",") + ";" + #LF$)
                EndIf
                
                WriteString(#PHF_Out, "}" + #LF$)
              Next i
              WriteString(#PHF_Out, "a:link { color:inherit; text-decoration:underline; }" + #LF$)
              WriteString(#PHF_Out, "a:visited { color:inherit; text-decoration:underline; }" + #LF$)
              WriteString(#PHF_Out, "a:hover { color:" + CSSColor(Style(#PHS_Keyword)\Color) + "; text-decoration:underline; }" + #LF$)
              WriteString(#PHF_Out, "a:active { color:" + CSSColor(Style(#PHS_Keyword)\Color) + "; text-decoration:underline; }" + #LF$)
              WriteString(#PHF_Out, "</style>" + #LF$)
              
              ; Generate body header
              WriteString(#PHF_Out, "</head>" + #LF$)
              WriteString(#PHF_Out, "<body>")
              CompilerIf (Not #PH_CompactSpaces)
                WriteString(#PHF_Out,  #LF$ + #LF$)
              CompilerEndIf
            Else
              ;? partial HTML (paragraph with border?)
            EndIf
            
            ; Call Syntax Highlighting library
            CompilerSelect (#PB_Compiler_OS)
              CompilerCase (#PB_OS_Windows)
                CallFunction(0, "SyntaxHighlight", *InBuffer, InLength, @HTMLCallback(), 0)
              CompilerDefault
                ;?
            CompilerEndSelect
          
            ; Finish full HTML page
            If (Config\FullPage)
            
              ; Remove extra linefeed
              If (Config\Hidden)
                CSS = #LF$ + "</span>"
                i = Len(CSS)
                Temp = Space(i)
                FileSeek(#PHF_Out, Loc(#PHF_Out) - i)
                ReadData(#PHF_Out, @Temp, i)
                If (Temp = CSS)
                  FileSeek(#PHF_Out, Loc(#PHF_Out) - i)
                  WriteString(#PHF_Out, "</span>")
                EndIf
              EndIf
              
              ; Generate HTML footer
              WriteString(#PHF_Out, "</body>" + #LF$)
              WriteString(#PHF_Out, "</html>")
            Else
              ;? partial HTML
            EndIf
          
            Result = #True
          EndIf
          FreeMemory(*InBuffer)
        EndIf
      EndIf
      CloseFile(#PHF_Out)
    Else
    EndIf
    CloseFile(#PHF_In)
  Else
  EndIf
  
  ProcedureReturn (Result)
EndProcedure



; Search the specified folder and highlight all source files
Procedure.i HighlightFolder(Path.s)
  Protected Result.i = #False
  NewList InFile.s()
  
  Config\SuccessFiles = ""
  Config\FailedFiles  = ""
  If (ExamineDirectory(0, Path, ""))
    While (NextDirectoryEntry(0))
      If (DirectoryEntryType(0) = #PB_DirectoryEntry_File)
        Select (LCase(GetExtensionPart(DirectoryEntryName(0))))
          Case "pb", "pbi"
            AddElement(InFile())
            InFile() = Path + DirectoryEntryName(0)
        EndSelect
      EndIf
    Wend
    FinishDirectory(0)
    If (ListSize(InFile()) > 0)
      Result = #True
      
      ForEach (InFile())
        Config\InFile  = InFile()
        Config\OutFile = InFile() + ".htm"
        If (HighlightFile(Config\InFile, Config\OutFile))
          Config\SuccessFiles + Config\OutFile + #LF$
        Else
          Config\FailedFiles + Config\OutFile + #LF$
          Result = #False
        EndIf
      Next
      
      Config\SuccessFiles = RTrim(Config\SuccessFiles, #LF$)
      Config\FailedFiles  = RTrim(Config\FailedFiles, #LF$)
      If (Not Result)
        If (Config\SuccessFiles)
          Warn("Generated:" + #LF$ + Config\SuccessFiles + #LF$ + #LF$ + "Failed:" + #LF$ + Config\FailedFiles)
        Else
          Warn("Failed to generate:" + #LF$ + Config\FailedFiles)
        EndIf
      EndIf
      
    Else
      Warn("No PureBasic files found!")
    EndIf
  Else
    Warn("Input path could not be found!")
  EndIf
  
  Config\OutFile = ""
  
  ProcedureReturn (Result)
EndProcedure





; Validate the form and begin file/folder highlight
Procedure.i TryHighlight()
  Protected Result.i = #True
  Protected Valid.i  = #True
  
  Config\InFile   = GetGadgetText(#PHG_InFile)
  Config\OutFile  = GetGadgetText(#PHG_OutFile)
  Config\Folder   = GetGadgetState(#PHG_InPathOption)
  Config\InPath   = GetGadgetText(#PHG_InPath)
    Config\InPath = RTrim(Config\InPath, #PS$)
  
  Config\FullPage     = #True
  Config\FontName     =  GetGadgetText(#PHG_FontName)
  Config\FontSize     =  Val(GetGadgetText(#PHG_FontSize))
  Config\BGColor      =  GetGadgetData(#PHG_BGColor)
  Config\TabLength    =  Val(GetGadgetText(#PHG_TabLength))
  Config\LineNumbers  =  GetGadgetState(#PHG_LineNumbers)
  Config\LNColor      =  GetGadgetData(#PHG_LNColor)
  Config\HideIDE      =  GetGadgetState(#PHG_HideIDE)
  Config\IncludeLinks =  GetGadgetState(#PHG_Links)
  Config\OpenDone     =  GetGadgetState(#PHG_OpenDone)
  
  If (Config\Folder)
    If (Config\InPath)
      Valid = #True
    Else
      Warn("Select a valid input folder.")
      SetActiveGadget(#PHG_InPath)
      Valid = #False
    EndIf
  Else
    If (Config\InFile)
      If (Config\OutFile)
        Valid = #True
      Else
        Warn("Select a valid output file.")
        SetActiveGadget(#PHG_OutFile)
        Valid = #False
      EndIf
    Else
      Warn("Select a valid input file.")
      SetActiveGadget(#PHG_InFile)
      Valid = #False
    EndIf
  EndIf
  
  If (Valid)
    If (Config\Folder)
      Result = HighlightFolder(Config\InPath + #PS$)
      If (Result)
        If (Config\OpenDone)
          RunProgram(Config\InPath + #PS$)
        Else
          Info("Successfully generated:" + #LF$ + Config\SuccessFiles)
        EndIf
      Else
        ; handled in HighlightFolder(), maybe should be here
      EndIf
    Else
      Result = HighlightFile(Config\InFile, Config\OutFile)
      If (Result)
        If (Config\OpenDone)
          RunProgram(Config\OutFile)
        Else
          Info("Successfully generated:" + #LF$ + Config\OutFile)
        EndIf
      Else
        Warn("Failed to generate:" + #LF$ + Config\OutFile)
      EndIf
    EndIf
  EndIf
  
  ProcedureReturn (Result)
EndProcedure

;-
;  EOF
; IDE Options = PureBasic 5.10 Beta 8 (Windows - x86)
; CursorPosition = 41
; FirstLine = 21
; Folding = --
; EnableXP
; EnableUser