; +-----------------+
; | PureHTML Shared |
; +-----------------+

;-
;- Structures

Structure PH_CONFIG
  InFile.s
  OutFile.s
  InPath.s
  LibFile.s
  OpenDone.i
  
  Folder.i
  FullPage.i
  FontName.s
  FontSize.i
  LineNumbers.i
  BGColor.i
  LNColor.i
  IncludeLinks.i
  TabLength.i
  HideIDE.i
  Hidden.i
  
  SuccessFiles.s
  FailedFiles.s
  IOPath.s
  
  LineCount.i
  LineTotal.i
  LineDigits.i
EndStructure

Structure PH_STYLE
  Color.i
  Bold.i
  Italic.i
  Underline.i
  Strike.i
EndStructure





;- Constants

; Exit Flags
Enumeration
  #PHE_Active = 0
  #PHE_Quit   = 2
  #PHE_Kill   = 4
EndEnumeration

; File Constants
#PH_PrefFile      = "PureHTML.ini"
#PH_SourcePattern = "PureBasic Code|*.pb;*.pbi|All Files (*.*)|*.*"
#PH_OutputPattern = "HTML File|*.htm;*.html|All Files (*.*)|*.*"

; Unicode not enabled
CompilerIf (#PB_Compiler_Unicode)
  CompilerError "Unicode is currently not supported"
CompilerEndIf

; Style IDs
Enumeration
  #PHS_Text
  #PHS_Keyword
  #PHS_Comment
  #PHS_Constant
  #PHS_String
  #PHS_Function
  #PHS_Asm
  #PHS_Operator
  #PHS_Structure
  #PHS_Number
  #PHS_Pointer
  #PHS_Separator
  #PHS_Label
  #PHS_LineNumber
  ;
  #PHS_Count
EndEnumeration











; Style Names
Global Dim StyleName.s(#PHS_Count - 1)
  StyleName( 0) = "Text"
  StyleName( 1) = "Keyword"
  StyleName( 2) = "Comment"
  StyleName( 3) = "Constant"
  StyleName( 4) = "String"
  StyleName( 5) = "Function"
  StyleName( 6) = "ASM"
  StyleName( 7) = "Operator"
  StyleName( 8) = "Structure"
  StyleName( 9) = "Number"
  StyleName(10) = "Pointer"
  StyleName(11) = "Separator"
  StyleName(12) = "Label"
  StyleName(13) = "Line Numbers"
  
; Style Keywords
Global Dim StyleKeyword.s(#PHS_Count - 1)
  StyleKeyword( 0) = "NormalText"
  StyleKeyword( 1) = "BasicKeyword"
  StyleKeyword( 2) = "Comment"
  StyleKeyword( 3) = "Constant"
  StyleKeyword( 4) = "String"
  StyleKeyword( 5) = "PureKeyword"
  StyleKeyword( 6) = "ASMKeyword"
  StyleKeyword( 7) = "Operator"
  StyleKeyword( 8) = "Structure"
  StyleKeyword( 9) = "Number"
  StyleKeyword(10) = "Pointer"
  StyleKeyword(11) = "Separator"
  StyleKeyword(12) = "Label"
  StyleKeyword(13) = "LineNumber"

; Short Style Names
Global Dim StyleRef.s(#PHS_Count - 1)
  StyleRef( 0) = "s0";"text"
  StyleRef( 1) = "s1";"keyword"
  StyleRef( 2) = "s2";"comment"
  StyleRef( 3) = "s3";"constant"
  StyleRef( 4) = "s4";"string"
  StyleRef( 5) = "s5";"function"
  StyleRef( 6) = "s6";"asm"
  StyleRef( 7) = "s7";"operator"
  StyleRef( 8) = "s8";"structure"
  StyleRef( 9) = "s9";"number"
  StyleRef(10) = "sA";"pointer"
  StyleRef(11) = "sB";"separator"
  StyleRef(12) = "sC";"label"
  ;
  StyleRef(#PHS_LineNumber) = "LN"

; Style Definitions
Global Dim Style.PH_STYLE(#PHS_Count - 1)

; Style Default Colors
Global Dim DefColor.i(#PHS_Count - 1)
  DefColor( 0) = $000000
  DefColor( 1) = $666600
  DefColor( 2) = $AAAA00
  DefColor( 3) = $724B92
  DefColor( 4) = $000000
  DefColor( 5) = $666600
  DefColor( 6) = $724B92
  DefColor( 7) = $000000
  DefColor( 8) = $000000
  DefColor( 9) = $000000
  DefColor(10) = $000000
  DefColor(11) = $000000
  DefColor(12) = $000000
  DefColor(13) = $808080








;- OS-Dependent

CompilerSelect (#PB_Compiler_OS)
  CompilerCase (#PB_OS_Windows)
    #PH_IconError = #MB_ICONHAND
    #PH_IconWarn  = #MB_ICONWARNING
    #PH_IconInfo  = #MB_ICONINFORMATION
    #PS$ = "\"
    
    Macro WLMO(Windows, Linux, Mac, Other)
      Windows
    EndMacro
    
  CompilerCase (#PB_OS_MacOS)
    #PH_IconError = #Null
    #PH_IconWarn  = #Null
    #PH_IconInfo  = #Null
    #PS$ = "/"
    
    Macro WLMO(Windows, Linux, Mac, Other)
      Mac
    EndMacro
    
  CompilerDefault
    CompilerError "This OS is not yet supported"
CompilerEndSelect






;- Variables

EnableExplicit

Global ExitFlag.i = #PHE_Active
Global Config.PH_CONFIG
Define Event.i






;-
;- Macros

Macro Warn(Message)
  MessageRequester(#PH_Title, Message, #PH_IconWarn)
EndMacro
Macro Info(Message)
  MessageRequester(#PH_Title, Message, #PH_IconInfo)
EndMacro


;-
;- Procedures

Procedure Shutdown(ExitCode.i = 0)
  End ExitCode
EndProcedure

Procedure Error(Message.s)
  MessageRequester(#PH_Title + " - Error", Message, #PH_IconError)
  Shutdown(1)
EndProcedure

Procedure.s VersionString(Number.i)
  Protected Result.s
  
  If (Number > 0)
    Result = StrF(Number * 0.01, 2)
  Else
    Result = "?"
  EndIf
  
  ProcedureReturn (Result)
EndProcedure

Procedure About()
  Protected St.s
  St = #PH_Title + #LF$ + "Version " + VersionString(#PH_Version)
  St + #LF$ + "  by " + #PH_Author + #LF$ + "  " + #PH_Website
  Info(St)
EndProcedure

;- Declares

Declare.i TryHighlight()
Declare.i TryImport()
Declare.i TryExport()

;-
;  EOF
; IDE Options = PureBasic 5.00 (Windows - x86)
; CursorPosition = 28
; FirstLine = 3
; Folding = --
; EnableXP
; EnableUser